/*-----------------------------------------------------------------------/
/  Low level disk interface module include file   (C)ChaN, 2014          /
/-----------------------------------------------------------------------*/

#ifndef _MMCPIC32_DEFINED
#define _MMCPIC32_DEFINED

#ifdef __cplusplus
extern "C" {
#endif


// ***************************************************
// ** CHANGE THE BELOW SETTINGS TO MATCH YOUR BOARD **
// ***************************************************
// SD card port and pin settings
#define CS_PORT A                   // Port on which CS is to be found, A - H
#define CS_PIN 0                   // Pin number of CS, 0 -15
#define SDO_PORT B                  // Port on which SDO/MOSI is to be found, A - H
#define SDO_PIN 5                   // Pin number of SDO/MOSI, 0 - 15

// SPI channel configuration
#define SPI_CHANNEL 2               // Channel number to use for SD card

typedef unsigned char BYTE;      // 8-bit
typedef unsigned int UINT;       // 32-bit
typedef unsigned long int DWORD; // 64-bit
typedef BYTE	DSTATUS;         // Status of disk functions

/* Results of Disk Functions */
typedef enum {
	RES_OK = 0,		/* 0: Successful */
	RES_ERROR,		/* 1: R/W Error */
	RES_WRPRT,		/* 2: Write Protected */
	RES_NOTRDY,		/* 3: Not Ready */
	RES_PARERR		/* 4: Invalid Parameter */
} DRESULT;

/*---------------------------------------*/
/* Prototypes for disk control functions */
DSTATUS disk_initialize (BYTE pdrv);
DSTATUS disk_status (BYTE pdrv);
DRESULT disk_read (BYTE pdrv, BYTE* buff, DWORD sector, UINT count);
DRESULT disk_write (BYTE pdrv, const BYTE* buff, DWORD sector, UINT count);
DRESULT disk_ioctl (BYTE pdrv, BYTE cmd, void* buff);
DRESULT SD_start_multi_block_read(DWORD sector);
DRESULT SD_read_multi_block(BYTE* buff, UINT cnt);
DRESULT SD_stop_multi_block_read();

/* Disk Status Bits (DSTATUS) */
#define STA_NOINIT		0x01	/* Drive not initialized */
#define STA_NODISK		0x02	/* No medium in the drive */
#define STA_PROTECT		0x04	/* Write protected */

/* Command codes for disk_ioctrl functions */
/* Generic command (Used by FatFs) */
#define CTRL_SYNC			0	/* Complete pending write process (needed at FF_FS_READONLY == 0) */
#define GET_SECTOR_COUNT	1	/* Get media size (needed at FF_USE_MKFS == 1) */
#define GET_SECTOR_SIZE		2	/* Get sector size (needed at FF_MAX_SS != FF_MIN_SS) */
#define GET_BLOCK_SIZE		3	/* Get erase block size (needed at FF_USE_MKFS == 1) */
#define CTRL_TRIM			4	/* Inform device that the data on the block of sectors is no longer used (needed at FF_USE_TRIM == 1) */

/* Generic command (Not used by FatFs) */
#define CTRL_POWER			5	/* Get/Set power status */
#define CTRL_LOCK			6	/* Lock/Unlock media removal */
#define CTRL_EJECT			7	/* Eject media */
#define CTRL_FORMAT			8	/* Create physical format on the media */

/* MMC/SDC specific ioctl command */
#define MMC_GET_TYPE		10	/* Get card type */
#define MMC_GET_CSD			11	/* Get CSD */
#define MMC_GET_CID			12	/* Get CID */
#define MMC_GET_OCR			13	/* Get OCR */
#define MMC_GET_SDSTAT		14	/* Get SD status */
#define ISDIO_READ			55	/* Read data form SD iSDIO register */
#define ISDIO_WRITE			56	/* Write data to SD iSDIO register */
#define ISDIO_MRITE			57	/* Masked write data to SD iSDIO register */

/* ATA/CF specific ioctl command */
#define ATA_GET_REV			20	/* Get F/W revision */
#define ATA_GET_MODEL		21	/* Get model name */
#define ATA_GET_SN			22	/* Get serial number */

// Definitions for MMC/SDC command
#define CMD0   (0)			// GO_IDLE_STATE 
#define CMD1   (1)			// SEND_OP_COND 
#define ACMD41 (41|0x80)	// SEND_OP_COND (SDC) 
#define CMD8   (8)			// SEND_IF_COND 
#define CMD9   (9)			// SEND_CSD 
#define CMD10  (10)			// SEND_CID 
#define CMD12  (12)			// STOP_TRANSMISSION 
#define ACMD13 (13|0x80)	// SD_STATUS (SDC) 
#define CMD16  (16)			// SET_BLOCKLEN 
#define CMD17  (17)			// READ_SINGLE_BLOCK 
#define CMD18  (18)			// READ_MULTIPLE_BLOCK 
#define CMD23  (23)			// SET_BLOCK_COUNT 
#define ACMD23 (23|0x80)	// SET_WR_BLK_ERASE_COUNT (SDC) 
#define CMD24  (24)			// WRITE_BLOCK 
#define CMD25  (25)			// WRITE_MULTIPLE_BLOCK 
#define CMD41  (41)			// SEND_OP_COND (ACMD) 
#define CMD55  (55)			// APP_CMD 
#define CMD58  (58)			// READ_OCR 

// MMC card type flags (MMC_GET_TYPE)
#define CT_MMC		0x01                // MMC ver 3 
#define CT_SD1		0x02                // SD ver 1 
#define CT_SD2		0x04                // SD ver 2 
#define CT_SDC		(CT_SD1 | CT_SD2)	// SD 
#define CT_BLOCK	0x08                // Block addressing 

// The following defines will be used to allow for simple configuration of the above settings
#define DEF_MERGER_INDIRECTOR(w,x,y,z) w ## x ## y ## z
#define DEF_MERGER(w,x,y,z) DEF_MERGER_INDIRECTOR(w,x,y,z)
#define CS_SETOUT() DEF_MERGER(TRIS,CS_PORT,CLR,) = 1 << CS_PIN
#define CS_LOW() DEF_MERGER(LAT,CS_PORT,CLR,) = 1 << CS_PIN
#define CS_HIGH() DEF_MERGER(LAT,CS_PORT,SET,) = 1 << CS_PIN
#define SDO_HIGH() DEF_MERGER(LAT,SDO_PORT,SET,) = 1 << SDO_PIN
#define SPI_CHAN_FUNC(suffix) DEF_MERGER(SPI, SPI_CHANNEL, suffix,)
#define SPI_CHAN_FUNC_PREFIXED(prefix, suffix) DEF_MERGER(prefix, SPI, SPI_CHANNEL, suffix)

// Change the SPI port number as needed on the following 5 lines

#define	FCLK_SLOW()	SPI_CHAN_FUNC(BRG) = 128	// Set slow clock (100k-400k) 
#define	FCLK_FAST()	SPI_CHAN_FUNC(BRG) = 0		// Set fast clock (depends on the CSD) 

// Status variables for SD card
static volatile DSTATUS Stat = STA_NOINIT;	/* Disk status */
static UINT CardType = 0;

#define xmit_spi(dat) 	xchg_spi(dat)
#define rcvr_spi()		xchg_spi(0xFF)
#define rcvr_spi_m(p)	SPI_CHAN_FUNC(BUF) = 0xFF; while (SPI_CHAN_FUNC(STATbits).RXBUFELM < 1); *(p) = (BYTE)SPI_CHAN_FUNC(BUF);

#ifdef __cplusplus
}
#endif

#endif